@@ -19,9 +19,8 @@
 
 #include <gst/gst.h>
 #include <telepathy-glib/telepathy-glib.h>
-#include <extensions/extensions.h>
-#include <gst/farsight/fs-element-added-notifier.h>
-#include <gst/farsight/fs-utils.h>
+#include <farstream/fs-element-added-notifier.h>
+#include <farstream/fs-utils.h>
 #include <telepathy-farstream/telepathy-farstream.h>
 
 typedef struct {
@@ -34,6 +33,9 @@ typedef struct {
   guint input_volume;
   guint output_volume;
 
+  gboolean has_audio_src;
+  gboolean has_video_src;
+
   GstElement *video_input;
   GstElement *video_capsfilter;
 
@@ -232,7 +234,7 @@ on_audio_input_volume_changed (TfContent *content,
   GstElement *volume;
   guint input_volume = 0;
 
-  g_object_get (content, "input-volume", &input_volume, NULL);
+  g_object_get (content, "request-input-volume", &input_volume, NULL);
 
   if (input_volume == 0)
     return;
@@ -247,7 +249,7 @@ setup_audio_source (ChannelContext *context, TfContent *content)
 {
   GstElement *result;
   GstElement *volume;
-  guint input_volume = 0;
+  gint input_volume = 0;
 
   result = gst_parse_bin_from_description (
       "pulsesrc ! audio/x-raw-int,rate=8000 ! queue"
@@ -255,18 +257,21 @@ setup_audio_source (ChannelContext *context, TfContent *content)
       " ! volume name=input_volume ! audioconvert ",
       TRUE, NULL);
 
+  /* FIXME Need to handle both requested/reported */
+  /* TODO Volume control should be handled in FsIo */
   g_object_get (content,
-      "input-volume", &input_volume,
+      "requested-input-volume", &input_volume,
       NULL);
 
-  if (input_volume != 0)
+  if (input_volume >= 0)
     {
       volume = gst_bin_get_by_name (GST_BIN (result), "input_volume");
+      g_debug ("Requested volume is: %i", input_volume);
       g_object_set (volume, "volume", (double)input_volume / 255.0, NULL);
       gst_object_unref (volume);
     }
 
-  g_signal_connect (content, "notify::input-volume",
+  g_signal_connect (content, "notify::request-input-volume",
       G_CALLBACK (on_audio_input_volume_changed),
       context);
 
@@ -276,19 +281,17 @@ setup_audio_source (ChannelContext *context, TfContent *content)
 static GstElement *
 setup_video_source (ChannelContext *context, TfContent *content)
 {
-  GstElement *result, *input, *rate, *scaler, *colorspace, *capsfilter;
+  GstElement *result, *capsfilter;
   GstCaps *caps;
   guint framerate = 0, width = 0, height = 0;
-  GstPad *pad, *ghost;
 
-  result = gst_bin_new ("video_input");
-  input = gst_element_factory_make ("autovideosrc", NULL);
-  rate = gst_element_factory_make ("videomaxrate", NULL);
-  scaler = gst_element_factory_make ("videoscale", NULL);
-  colorspace = gst_element_factory_make ("colorspace", NULL);
-  capsfilter = gst_element_factory_make ("capsfilter", NULL);
+  result = gst_parse_bin_from_description_full (
+      "autovideosrc ! videomaxrate ! videoscale ! colorspace ! capsfilter name=c",
+      TRUE, NULL, GST_PARSE_FLAG_FATAL_ERRORS, NULL);
+
+  g_assert (result);
+  capsfilter = gst_bin_get_by_name (GST_BIN (result), "c");
 
-  g_assert (input && rate && scaler && colorspace && capsfilter);
   g_object_get (content,
       "framerate", &framerate,
       "width", &width,
@@ -316,18 +319,7 @@ setup_video_source (ChannelContext *context, TfContent *content)
 
   g_object_set (G_OBJECT (capsfilter), "caps", caps, NULL);
 
-  gst_bin_add_many (GST_BIN (result), input, rate, scaler,
-      colorspace, capsfilter, NULL);
-  g_assert (gst_element_link_many (input, rate, scaler,
-      colorspace, capsfilter, NULL));
-
-  pad = gst_element_get_static_pad (capsfilter, "src");
-  g_assert (pad != NULL);
-
-  ghost = gst_ghost_pad_new ("src", pad);
-  gst_element_add_pad (result, ghost);
-
-  g_object_unref (pad);
+  gst_caps_unref (caps);
 
   context->video_input = result;
   context->video_capsfilter = capsfilter;
@@ -343,19 +335,17 @@ setup_video_source (ChannelContext *context, TfContent *content)
   return result;
 }
 
-
-static void
-content_added_cb (TfChannel *channel,
-    TfContent *content,
-    gpointer user_data)
+static gboolean
+start_sending_cb (TfContent *content, gpointer user_data)
 {
+  ChannelContext *context = user_data;
   GstPad *srcpad, *sinkpad;
   FsMediaType mtype;
   GstElement *element;
   GstStateChangeReturn ret;
-  ChannelContext *context = user_data;
+  gboolean res = FALSE;
 
-  g_debug ("Content added");
+  g_debug ("Start sending");
 
   g_object_get (content,
     "sink-pad", &sinkpad,
@@ -365,9 +355,15 @@ content_added_cb (TfChannel *channel,
   switch (mtype)
     {
       case FS_MEDIA_TYPE_AUDIO:
+        if (context->has_audio_src)
+          goto out;
+
         element = setup_audio_source (context, content);
         break;
       case FS_MEDIA_TYPE_VIDEO:
+        if (context->has_video_src)
+          goto out;
+
         element = setup_video_source (context, content);
         break;
       default:
@@ -375,8 +371,6 @@ content_added_cb (TfChannel *channel,
         goto out;
     }
 
-  g_signal_connect (content, "src-pad-added",
-    G_CALLBACK (src_pad_added_cb), context);
 
   gst_bin_add (GST_BIN (context->pipeline), element);
   srcpad = gst_element_get_pad (element, "src");
@@ -385,7 +379,7 @@ content_added_cb (TfChannel *channel,
     {
       tp_channel_close_async (TP_CHANNEL (context->proxy), NULL, NULL);
       g_warning ("Couldn't link source pipeline !?");
-      return;
+      goto out2;
     }
 
   ret = gst_element_set_state (element, GST_STATE_PLAYING);
@@ -393,12 +387,32 @@ content_added_cb (TfChannel *channel,
     {
       tp_channel_close_async (TP_CHANNEL (context->proxy), NULL, NULL);
       g_warning ("source pipeline failed to start!?");
-      return;
+      goto out2;
     }
 
+  res = TRUE;
+
+out2:
   g_object_unref (srcpad);
 out:
   g_object_unref (sinkpad);
+
+  return res;
+}
+
+static void
+content_added_cb (TfChannel *channel,
+    TfContent *content,
+    gpointer user_data)
+{
+  ChannelContext *context = user_data;
+
+  g_debug ("Content added");
+
+  g_signal_connect (content, "src-pad-added",
+    G_CALLBACK (src_pad_added_cb), context);
+  g_signal_connect (content, "start-sending",
+      G_CALLBACK (start_sending_cb), context);
 }
 
 static void
@@ -430,6 +444,19 @@ conference_added_cb (TfChannel *channel,
   gst_element_set_state (conference, GST_STATE_PLAYING);
 }
 
+
+static void
+conference_removed_cb (TfChannel *channel,
+  GstElement *conference,
+  gpointer user_data)
+{
+  ChannelContext *context = user_data;
+
+  gst_element_set_locked_state (conference, TRUE);
+  gst_element_set_state (conference, GST_STATE_NULL);
+  gst_bin_remove (GST_BIN (context->pipeline), conference);
+}
+
 static gboolean
 dump_pipeline_cb (gpointer data)
 {
@@ -448,17 +475,16 @@ new_tf_channel_cb (GObject *source,
   gpointer user_data)
 {
   ChannelContext *context = user_data;
+  GError *error = NULL;
 
   g_debug ("New TfChannel");
 
-  context->channel = TF_CHANNEL (g_async_initable_new_finish (
-      G_ASYNC_INITABLE (source), result, NULL));
-
+  context->channel = tf_channel_new_finish (source, result, &error);
 
   if (context->channel == NULL)
     {
-      g_warning ("Failed to create channel");
-      return;
+      g_error ("Failed to create channel: %s", error->message);
+      g_clear_error (&error);
     }
 
   g_debug ("Adding timeout");
@@ -467,6 +493,10 @@ new_tf_channel_cb (GObject *source,
   g_signal_connect (context->channel, "fs-conference-added",
     G_CALLBACK (conference_added_cb), context);
 
+
+  g_signal_connect (context->channel, "fs-conference-removed",
+    G_CALLBACK (conference_removed_cb), context);
+
   g_signal_connect (context->channel, "content-added",
     G_CALLBACK (content_added_cb), context);
 }
@@ -543,7 +573,7 @@ new_call_channel_cb (TpSimpleHandler *handler,
 
   tp_handle_channels_context_accept (handler_context);
 
-  tf_future_cli_channel_type_call_call_accept (proxy, -1,
+  tp_cli_channel_type_call_call_accept (proxy, -1,
       NULL, NULL, NULL, NULL);
 
   context->proxy = g_object_ref (proxy);
@@ -556,17 +586,16 @@ int
 main (int argc, char **argv)
 {
   TpBaseClient *client;
-  TpAccountManager *account_manager;
+  TpAccountManager *am;
 
   g_type_init ();
-  tf_init ();
   gst_init (&argc, &argv);
 
   loop = g_main_loop_new (NULL, FALSE);
 
-  account_manager = tp_account_manager_dup ();
+  am = tp_account_manager_dup ();
 
-  client = tp_simple_handler_new_with_am (account_manager,
+  client = tp_simple_handler_new_with_am (am,
     FALSE,
     FALSE,
     "TpFsCallHandlerDemo",
@@ -578,35 +607,35 @@ main (int argc, char **argv)
   tp_base_client_take_handler_filter (client,
     tp_asv_new (
        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-          TF_FUTURE_IFACE_CHANNEL_TYPE_CALL,
+          TP_IFACE_CHANNEL_TYPE_CALL,
        TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT,
           TP_HANDLE_TYPE_CONTACT,
-        TF_FUTURE_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, G_TYPE_BOOLEAN,
+        TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, G_TYPE_BOOLEAN,
           TRUE,
        NULL));
 
   tp_base_client_take_handler_filter (client,
     tp_asv_new (
        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-          TF_FUTURE_IFACE_CHANNEL_TYPE_CALL,
+          TP_IFACE_CHANNEL_TYPE_CALL,
        TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT,
           TP_HANDLE_TYPE_CONTACT,
-        TF_FUTURE_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, G_TYPE_BOOLEAN,
+        TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, G_TYPE_BOOLEAN,
           TRUE,
        NULL));
 
   tp_base_client_add_handler_capabilities_varargs (client,
-    TF_FUTURE_IFACE_CHANNEL_TYPE_CALL "/video/h264",
-    TF_FUTURE_IFACE_CHANNEL_TYPE_CALL "/shm",
-    TF_FUTURE_IFACE_CHANNEL_TYPE_CALL "/ice",
-    TF_FUTURE_IFACE_CHANNEL_TYPE_CALL "/gtalk-p2p",
+    TP_IFACE_CHANNEL_TYPE_CALL "/video/h264",
+    TP_IFACE_CHANNEL_TYPE_CALL "/shm",
+    TP_IFACE_CHANNEL_TYPE_CALL "/ice",
+    TP_IFACE_CHANNEL_TYPE_CALL "/gtalk-p2p",
     NULL);
 
   tp_base_client_register (client, NULL);
 
   g_main_loop_run (loop);
 
-  g_object_unref (account_manager);
+  g_object_unref (am);
   g_object_unref (client);
   g_main_loop_unref (loop);
 
