@@ -24,7 +24,7 @@
  *
  * This class handles the
  * org.freedesktop.Telepathy.Channel.Interface.Call on a
- * channel using Farsight2.
+ * channel using Farstream.
  */
 
 
@@ -32,12 +32,10 @@
 
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/interfaces.h>
-#include <gst/farsight/fs-conference-iface.h>
-
-#include "extensions/extensions.h"
+#include <farstream/fs-conference.h>
 
 #include "call-content.h"
-#include "tf-signals-marshal.h"
+#include "call-priv.h"
 
 
 static void call_channel_async_initable_init (GAsyncInitableIface *asynciface);
@@ -94,8 +92,14 @@ static gboolean tf_call_channel_init_finish (GAsyncInitable *initable,
     GAsyncResult *res,
     GError **error);
 
-static void got_hardware_streaming (TpProxy *proxy, const GValue *out_value,
-    const GError *error, gpointer user_data, GObject *weak_object);
+static void content_added (TpCallChannel *proxy,
+    TpCallContent *context_proxy, TfCallChannel *self);
+static void content_removed (TpCallChannel *proxy,
+    TpCallContent *content_proxy, TpCallStateReason *reason,
+    TfCallChannel *self);
+static void channel_prepared (GObject *proxy, GAsyncResult *prepare_res,
+    gpointer user_data);
+
 
 
 static void
@@ -108,37 +112,33 @@ tf_call_channel_class_init (TfCallChannelClass *klass)
 
   g_object_class_install_property (object_class, PROP_FS_CONFERENCES,
       g_param_spec_boxed ("fs-conferences",
-          "Farsight2 FsConference object",
-          "GPtrArray of Farsight2 FsConferences for this channel",
+          "Farstream FsConference object",
+          "GPtrArray of Farstream FsConferences for this channel",
           G_TYPE_PTR_ARRAY,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   signals[SIGNAL_FS_CONFERENCE_ADDED] = g_signal_new ("fs-conference-added",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, FS_TYPE_CONFERENCE);
 
   signals[SIGNAL_FS_CONFERENCE_REMOVED] = g_signal_new ("fs-conference-removed",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, FS_TYPE_CONFERENCE);
 
   signals[SIGNAL_CONTENT_ADDED] = g_signal_new ("content-added",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, TF_TYPE_CALL_CONTENT);
 
   signals[SIGNAL_CONTENT_REMOVED] = g_signal_new ("content-removed",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, TF_TYPE_CALL_CONTENT);
 }
 
@@ -200,10 +200,12 @@ tf_call_channel_init_async (GAsyncInitable *initable,
   res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
       tf_call_channel_init_async);
 
-  tp_cli_dbus_properties_call_get (self->proxy, -1,
-      TF_FUTURE_IFACE_CHANNEL_TYPE_CALL,
-      "HardwareStreaming",
-      got_hardware_streaming, res, NULL, G_OBJECT (self));
+  tp_g_signal_connect_object (self->proxy, "content-added",
+      G_CALLBACK (content_added), self, 0);
+  tp_g_signal_connect_object (self->proxy, "content-removed",
+      G_CALLBACK (content_removed), self, 0);
+
+  tp_proxy_prepare_async (self->proxy, NULL, channel_prepared, res);
 }
 
 static gboolean
@@ -244,18 +246,9 @@ tf_call_channel_dispose (GObject *object)
      already been disposed of. */
   if (self->contents)
     {
-      GHashTableIter iter;
-      gpointer key, value;
-
-      g_hash_table_iter_init (&iter, self->contents);
-      while (g_hash_table_iter_next (&iter, &key, &value))
-        {
-          g_object_run_dispose (G_OBJECT (value));
-        }
-
-      g_hash_table_destroy (self->contents);
-      self->contents = NULL;
+      g_ptr_array_free (self->contents, TRUE);
     }
+  self->contents = NULL;
 
   if (self->participants)
     g_ptr_array_unref (self->participants);
@@ -317,38 +310,34 @@ content_ready (GObject *object, GAsyncResult *res, gpointer user_data)
   if (g_async_initable_init_finish (G_ASYNC_INITABLE (object), res, NULL))
     {
       g_signal_emit (self, signals[SIGNAL_CONTENT_ADDED], 0, content);
-      g_object_unref (content);
     }
   else
     {
-      GHashTableIter iter;
-      gpointer key, value;
-
-      g_hash_table_iter_init (&iter, self->contents);
-      while (g_hash_table_iter_next (&iter, &key, &value))
-        {
-          if (value == object)
-            {
-              g_hash_table_iter_remove (&iter);
-              break;
-            }
-        }
+      g_ptr_array_remove_fast (self->contents, content);
     }
 
   g_object_unref (self);
 }
 
 static gboolean
-add_content (TfCallChannel *self, const gchar *content_path)
+add_content (TfCallChannel *self, TpCallContent *content_proxy)
 {
   GError *error = NULL;
   TfCallContent *content;
+  guint i;
 
   /* Check if content already added */
-  if (g_hash_table_lookup (self->contents, content_path))
-    return TRUE;
+  if (!self->contents)
+    return FALSE;
 
-  content = tf_call_content_new_async (self, content_path,
+  for (i = 0; i < self->contents->len; i++)
+    {
+      if (tf_call_content_get_proxy (g_ptr_array_index (self->contents, i)) ==
+          content_proxy)
+        return TRUE;
+    }
+
+  content = tf_call_content_new_async (self, content_proxy,
       &error, content_ready, g_object_ref (self));
 
   if (error)
@@ -359,32 +348,92 @@ add_content (TfCallChannel *self, const gchar *content_path)
       return FALSE;
     }
 
-  g_hash_table_insert (self->contents, g_strdup (content_path), content);
+  g_ptr_array_add (self->contents, content);
 
   return TRUE;
 }
 
 static void
-got_contents (TpProxy *proxy, const GValue *out_value,
-    const GError *error, gpointer user_data, GObject *weak_object)
+content_added (TpCallChannel *proxy,
+    TpCallContent *content_proxy,
+    TfCallChannel *self)
+{
+  /* Ignore signals before we got the "Contents" property to avoid races that
+   * could cause the same content to be added twice
+   */
+
+  if (!self->contents)
+    return;
+
+  add_content (self, content_proxy);
+}
+
+static void
+content_removed (TpCallChannel *proxy,
+    TpCallContent *content_proxy,
+    TpCallStateReason *reason,
+    TfCallChannel *self)
+{
+  guint i;
+  if (!self->contents)
+    return;
+
+  for (i = 0; i < self->contents->len; i++)
+    {
+
+      if (tf_call_content_get_proxy (g_ptr_array_index (self->contents, i)) ==
+          content_proxy)
+        {
+          TfCallContent *content = g_ptr_array_index (self->contents, i);
+
+          g_object_ref (content);
+          g_ptr_array_remove_index_fast (self->contents, i);
+          g_signal_emit (self, signals[SIGNAL_CONTENT_REMOVED], 0, content);
+          g_object_unref (content);
+          return;
+        }
+    }
+}
+
+static void
+free_content (gpointer data)
+{
+  TfCallContent *content = data;
+
+  _tf_call_content_destroy (content);
+  g_object_unref (content);
+}
+
+static void
+channel_prepared (GObject *proxy, GAsyncResult *prepare_res, gpointer user_data)
 {
-  TfCallChannel *self = TF_CALL_CHANNEL (weak_object);
   GSimpleAsyncResult *res = user_data;
+  TfCallChannel *self =
+      TF_CALL_CHANNEL (g_async_result_get_source_object (G_ASYNC_RESULT (res)));
+  GError *error = NULL;
   GPtrArray *contents;
   guint i;
 
-  if (error)
+  if (!tp_proxy_prepare_finish (proxy, prepare_res, &error))
     {
-      g_warning ("Error getting the Contents property: %s",
+      g_warning ("Preparing the channel: %s",
           error->message);
-      g_simple_async_result_set_from_error (res, error);
+      g_simple_async_result_take_error (res, error);
       goto out;
     }
 
-  contents = g_value_get_boxed (out_value);
+  if (tp_call_channel_has_hardware_streaming (TP_CALL_CHANNEL (proxy)))
+    {
+      g_warning ("Hardware streaming property is TRUE, ignoring");
 
-  self->contents = g_hash_table_new_full (g_str_hash, g_str_equal,
-      g_free, g_object_unref);
+      g_simple_async_result_set_error (res, TP_ERROR, TP_ERROR_NOT_CAPABLE,
+          "This channel does hardware streaming, not handled here");
+      goto out;
+    }
+
+  contents = tp_call_channel_get_contents (TP_CALL_CHANNEL (proxy));
+
+  self->contents = g_ptr_array_new_with_free_func (free_content);
 
   for (i = 0; i < contents->len; i++)
     if (!add_content (self, g_ptr_array_index (contents, i)))
@@ -395,108 +444,7 @@ got_contents (TpProxy *proxy, const GValue *out_value,
 out:
   g_simple_async_result_complete (res);
   g_object_unref (res);
-}
-
-static void
-content_added (TpChannel *proxy,
-    const gchar *arg_Content,
-    gpointer user_data,
-    GObject *weak_object)
-{
-  TfCallChannel *self = TF_CALL_CHANNEL (weak_object);
-
-  /* Ignore signals before we got the "Contents" property to avoid races that
-   * could cause the same content to be added twice
-   */
-
-  if (!self->contents)
-    return;
-
-  add_content (self, arg_Content);
-}
-
-static void
-content_removed (TpChannel *proxy,
-    const gchar *arg_Content,
-    gpointer user_data,
-    GObject *weak_object)
-{
-  TfCallChannel *self = TF_CALL_CHANNEL (weak_object);
-  TfCallContent *content;
-
-  if (!self->contents)
-    return;
-
-  content = g_hash_table_lookup (self->contents, arg_Content);
-
-  if (content)
-    {
-      g_object_ref (content);
-      g_hash_table_remove (self->contents, arg_Content);
-
-      g_signal_emit (self, signals[SIGNAL_CONTENT_REMOVED], 0, content);
-      g_object_unref (content);
-    }
-}
-
-
-static void
-got_hardware_streaming (TpProxy *proxy, const GValue *out_value,
-    const GError *error, gpointer user_data, GObject *weak_object)
-{
-  TfCallChannel *self = TF_CALL_CHANNEL (weak_object);
-  GSimpleAsyncResult *res = user_data;
-  GError *myerror = NULL;
-
-  if (error)
-    {
-      g_warning ("Error getting the hardware streaming property: %s",
-          error->message);
-      g_simple_async_result_set_from_error (res, error);
-      goto error;
-    }
-
-  if (g_value_get_boolean (out_value))
-    {
-      g_warning ("Hardware streaming property is TRUE, ignoring");
-
-      g_simple_async_result_set_error (res, TP_ERROR, TP_ERROR_NOT_CAPABLE,
-          "This channel does hardware streaming, not handled here");
-      goto error;
-    }
-
-  tf_future_cli_channel_type_call_connect_to_content_added (TP_CHANNEL (proxy),
-      content_added, NULL, NULL, G_OBJECT (self), &myerror);
-  if (myerror)
-    {
-      g_warning ("Error connectiong to ContentAdded signal: %s",
-          myerror->message);
-      g_simple_async_result_set_from_error (res, myerror);
-      g_clear_error (&myerror);
-      goto error;
-    }
-
-  tf_future_cli_channel_type_call_connect_to_content_removed (
-      TP_CHANNEL (proxy), content_removed, NULL, NULL, G_OBJECT (self),
-      &myerror);
-  if (myerror)
-    {
-      g_warning ("Error connectiong to ContentRemoved signal: %s",
-          myerror->message);
-      g_simple_async_result_set_from_error (res, myerror);
-      g_clear_error (&myerror);
-      goto error;
-    }
-
-  tp_cli_dbus_properties_call_get (proxy, -1,
-      TF_FUTURE_IFACE_CHANNEL_TYPE_CALL, "Contents",
-      got_contents, res, NULL, G_OBJECT (self));
-
-  return;
-
-error:
-  g_simple_async_result_complete (res);
-  g_object_unref (res);
+  g_object_unref (self);
 }
 
 void
@@ -541,9 +489,8 @@ tf_call_channel_bus_message (TfCallChannel *channel,
 {
   GError *error = NULL;
   gchar *debug;
-  GHashTableIter iter;
-  gpointer key, value;
   struct CallConference *cc;
+  guint i;
 
   cc = find_call_conference_by_conference (channel, GST_MESSAGE_SRC (message));
   if (!cc)
@@ -573,9 +520,9 @@ tf_call_channel_bus_message (TfCallChannel *channel,
       break;
     }
 
-  g_hash_table_iter_init (&iter, channel->contents);
-  while (g_hash_table_iter_next (&iter, &key, &value))
-    if (tf_call_content_bus_message (value, message))
+  for (i = 0; i < channel->contents->len; i++)
+    if (tf_call_content_bus_message (g_ptr_array_index (channel->contents, i),
+            message))
       return TRUE;
 
   return FALSE;
@@ -584,9 +531,8 @@ tf_call_channel_bus_message (TfCallChannel *channel,
 void
 tf_call_channel_error (TfCallChannel *channel)
 {
-  tf_future_cli_channel_type_call_call_hangup (channel->proxy,
-      -1, TF_FUTURE_CALL_STATE_CHANGE_REASON_UNKNOWN, "", "",
-      NULL, NULL, NULL, NULL);
+  tp_call_channel_hangup_async (TP_CALL_CHANNEL (channel->proxy),
+      TP_CALL_STATE_CHANGE_REASON_UNKNOWN, "", "", NULL, NULL);
 }
 
 
@@ -684,7 +630,7 @@ _tf_call_channel_get_participant (TfCallChannel *channel,
         }
     }
 
-  p = fs_conference_new_participant (fsconference, NULL, error);
+  p = fs_conference_new_participant (fsconference, error);
   if (!p)
     return NULL;
 
@@ -719,5 +665,4 @@ _tf_call_channel_put_participant (TfCallChannel *channel,
           return;
         }
     }
-
 }
