@@ -25,26 +25,19 @@
  *
  * This class handles the
  * org.freedesktop.Telepathy.Channel.Interface.MediaSignalling on a
- * channel using Farsight2 or the media part of the
+ * channel using Farstream or the media part of the
  * org.freedesktop.Telepathy.Channel.Type.Call that has HardwareStreaming=FALSE
  */
 
 #include <stdlib.h>
 
-#include <telepathy-glib/channel.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/enums.h>
-#include <telepathy-glib/errors.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/util.h>
+#include <telepathy-glib/telepathy-glib.h>
 
-#include <gst/farsight/fs-conference-iface.h>
+#include <farstream/fs-conference.h>
 
-#include "extensions/extensions.h"
 
 #include "channel.h"
 #include "channel-priv.h"
-#include "tf-signals-marshal.h"
 #include "media-signalling-channel.h"
 #include "media-signalling-content.h"
 #include "call-channel.h"
@@ -152,8 +145,23 @@ tf_channel_get_property (GObject    *object,
       break;
     case PROP_FS_CONFERENCES:
       if (self->priv->call_channel)
-        g_object_get_property (G_OBJECT (self->priv->call_channel),
-            "fs-conferences", value);
+        {
+          g_object_get_property (G_OBJECT (self->priv->call_channel),
+              "fs-conferences", value);
+        }
+      else if (self->priv->media_signalling_channel &&
+               self->priv->media_signalling_channel->session)
+        {
+          GPtrArray *array =
+              g_ptr_array_new_with_free_func ((GDestroyNotify) gst_object_unref);
+          FsConference *conf = NULL;
+
+          g_object_get (self->priv->media_signalling_channel->session,
+              "farstream-conference", &conf, NULL);
+          g_ptr_array_add (array, conf);
+          g_value_take_boxed (value, array);
+        }
+
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -326,22 +334,34 @@ channel_prepared (GObject *obj,
           "stream-created", G_CALLBACK (channel_stream_created),
           self, 0);
       g_simple_async_result_set_op_res_gboolean (res, TRUE);
+      g_simple_async_result_complete (res);
     }
   else if (tp_proxy_has_interface_by_id (as_proxy,
-          TF_FUTURE_IFACE_QUARK_CHANNEL_TYPE_CALL))
+          TP_IFACE_QUARK_CHANNEL_TYPE_CALL))
     {
-      tf_call_channel_new_async (channel_proxy, call_channel_ready, res);
+      if (!TP_IS_CALL_CHANNEL (channel_proxy))
+        {
+          g_simple_async_result_set_error (res, TP_ERROR,
+              TP_ERROR_INVALID_ARGUMENT,
+              "You must pass a TpCallChannel object if its a Call channel");
+          g_simple_async_result_set_op_res_gboolean (res, FALSE);
+          g_simple_async_result_complete (res);
+        }
+      else
+        {
+          tf_call_channel_new_async (channel_proxy, call_channel_ready, res);
 
-      self->priv->channel_invalidated_handler = g_signal_connect (
-          self->priv->channel_proxy,
-          "invalidated", G_CALLBACK (channel_invalidated), self);
+          self->priv->channel_invalidated_handler = g_signal_connect (
+              self->priv->channel_proxy,
+              "invalidated", G_CALLBACK (channel_invalidated), self);
+        }
     }
   else
     {
       g_simple_async_result_set_error (res, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
           "Channel does not implement "
           TP_IFACE_CHANNEL_INTERFACE_MEDIA_SIGNALLING " or "
-          TF_FUTURE_IFACE_CHANNEL_TYPE_CALL);
+          TP_IFACE_CHANNEL_TYPE_CALL);
       goto error;
     }
 
@@ -462,8 +482,8 @@ tf_channel_class_init (TfChannelClass *klass)
 
   g_object_class_install_property (object_class, PROP_FS_CONFERENCES,
       g_param_spec_boxed ("fs-conferences",
-          "Farsight2 FsConferences objects",
-          "GPtrArray of Farsight2 FsConferences for this channel",
+          "Farstream FsConferences objects",
+          "GPtrArray of Farstream FsConferences for this channel",
           G_TYPE_PTR_ARRAY,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
@@ -480,8 +500,7 @@ tf_channel_class_init (TfChannelClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          g_cclosure_marshal_VOID__VOID,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 0);
 
   /**
@@ -496,8 +515,7 @@ tf_channel_class_init (TfChannelClass *klass)
   signals[SIGNAL_FS_CONFERENCE_ADDED] = g_signal_new ("fs-conference-added",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, FS_TYPE_CONFERENCE);
 
   /**
@@ -512,8 +530,7 @@ tf_channel_class_init (TfChannelClass *klass)
   signals[SIGNAL_FS_CONFERENCE_REMOVED] = g_signal_new ("fs-conference-removed",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, FS_TYPE_CONFERENCE);
 
 
@@ -532,8 +549,7 @@ tf_channel_class_init (TfChannelClass *klass)
   signals[SIGNAL_CONTENT_ADDED] = g_signal_new ("content-added",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, TF_TYPE_CONTENT);
 
   /**
@@ -547,8 +563,7 @@ tf_channel_class_init (TfChannelClass *klass)
   signals[SIGNAL_CONTENT_REMOVED] = g_signal_new ("content-removed",
       G_OBJECT_CLASS_TYPE (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL,
-      _tf_marshal_VOID__OBJECT,
+      0, NULL, NULL, NULL,
       G_TYPE_NONE, 1, TF_TYPE_CONTENT);
 }
 
@@ -597,7 +612,7 @@ channel_invalidated (TpChannel *channel_proxy,
  * Creates a new #TfChannel from an existing channel proxy, the new
  * TfChannel object will be return in the async callback.
  *
- * The user must call g_async_initable_new_finish() in the callback
+ * The user must call tf_channel_new_finish() in the callback
  * to get the finished object.
  */
 
@@ -606,6 +621,9 @@ tf_channel_new_async (TpChannel *channel_proxy,
     GAsyncReadyCallback callback,
     gpointer user_data)
 {
+  g_return_if_fail (channel_proxy != NULL);
+  g_return_if_fail (callback != NULL);
+
   return g_async_initable_new_async (TF_TYPE_CHANNEL,
       0, NULL, callback, user_data,
       "channel", channel_proxy,
@@ -627,6 +645,9 @@ gboolean
 tf_channel_bus_message (TfChannel *channel,
     GstMessage *message)
 {
+  g_return_val_if_fail (channel != NULL, FALSE);
+  g_return_val_if_fail (message != NULL, FALSE);
+
   if (channel->priv->media_signalling_channel)
     return tf_media_signalling_channel_bus_message (
         channel->priv->media_signalling_channel, message);
@@ -666,3 +687,24 @@ content_removed (GObject *proxy, TfContent *content, TfChannel *self)
 {
   g_signal_emit (self, signals[SIGNAL_CONTENT_REMOVED], 0, content);
 }
+
+/**
+ * tf_channel_new_finish:
+ * @object: The #GObject
+ * @result: a #GAsyncResult
+ * @error: the location of a #GError or %NULL to ignore it
+ *
+ * Completes the construction of a TfChannel.
+ *
+ * Returns: a #TfChannel or %NULL if there was an error
+ * Since: 0.2.2
+ */
+
+TfChannel *
+tf_channel_new_finish (GObject *object,
+    GAsyncResult *result,
+    GError **error)
+{
+  return (TfChannel *) g_async_initable_new_finish (G_ASYNC_INITABLE (object),
+      result, error);
+}
