@@ -2,10 +2,9 @@
 #include "content.h"
 #include "content-priv.h"
 
-#include <gst/farsight/fs-conference-iface.h>
+#include <farstream/fs-conference.h>
 
 #include "channel.h"
-#include "tf-signals-marshal.h"
 
 
 /**
@@ -15,10 +14,10 @@
  * Objects of this class allow the user to handle the media side of a Telepathy
  * channel handled by #TfChannel.
  *
- * This object is created by the #TfChannel and the user is notified of its
- * creation by the #TfChannel::content-added signal. In the callback for this
- * signal, the user should call tf_content_set_codec_preferences() and connect
- * to the #TfContent::src-pad-added signal.
+ * This object is created by the #TfChannel and the user is notified
+ * of its creation by the #TfChannel::content-added signal. In the
+ * callback for this signal, the user should connect to the
+ * #TfContent::src-pad-added signal.
  *
  */
 
@@ -74,22 +73,22 @@ tf_content_class_init (TfContentClass *klass)
   g_object_class_install_property (object_class, PROP_TF_CHANNEL,
       g_param_spec_object ("tf-channel",
           "Parent TfChannel object ",
-          "The Telepathy-Farsight Channel for this object",
+          "The Telepathy-Farstream Channel for this object",
           TF_TYPE_CHANNEL,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (object_class, PROP_FS_CONFERENCE,
       g_param_spec_object ("fs-conference",
-          "Farsight2 FsConference used by the Content ",
-          "The Farsight2 conference for this content "
+          "Farstream FsConference used by the Content ",
+          "The Farstream conference for this content "
           "(could be the same as other contents)",
           FS_TYPE_CONFERENCE,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (object_class, PROP_FS_SESSION,
       g_param_spec_object ("fs-session",
-          "Farsight2 FsSession ",
-          "The Farsight2 session for this content",
+          "Farstream FsSession ",
+          "The Farstream session for this content",
           FS_TYPE_SESSION,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
@@ -135,8 +134,7 @@ tf_content_class_init (TfContentClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          g_signal_accumulator_true_handled, NULL,
-          _tf_marshal_BOOLEAN__VOID,
+          g_signal_accumulator_true_handled, NULL, NULL,
           G_TYPE_BOOLEAN, 0);
 
   /**
@@ -151,8 +149,7 @@ tf_content_class_init (TfContentClass *klass)
       g_signal_new ("stop-sending",
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
-          0, NULL, NULL,
-          g_cclosure_marshal_VOID__VOID,
+          0, NULL, NULL, NULL,
           G_TYPE_NONE, 0);
 
   /**
@@ -174,8 +171,7 @@ tf_content_class_init (TfContentClass *klass)
       g_signal_new ("src-pad-added",
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
-          0, NULL, NULL,
-          _tf_marshal_VOID__UINT_OBJECT_OBJECT_BOXED,
+          0, NULL, NULL, NULL,
           G_TYPE_NONE, 4,
           G_TYPE_UINT, FS_TYPE_STREAM, GST_TYPE_PAD, FS_TYPE_CODEC);
 
@@ -202,8 +198,7 @@ tf_content_class_init (TfContentClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          g_signal_accumulator_true_handled, NULL,
-          _tf_marshal_BOOLEAN__POINTER_UINT,
+          g_signal_accumulator_true_handled, NULL, NULL,
           G_TYPE_BOOLEAN, 2, G_TYPE_POINTER, G_TYPE_UINT);
 
   /**
@@ -220,8 +215,7 @@ tf_content_class_init (TfContentClass *klass)
       g_signal_new ("stop-receiving",
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
-          0, NULL, NULL,
-          _tf_marshal_VOID__POINTER_UINT,
+          0, NULL, NULL, NULL,
           G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_UINT);
 
   /**
@@ -236,8 +230,7 @@ tf_content_class_init (TfContentClass *klass)
       g_signal_new ("restart-source",
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
-          0, NULL, NULL,
-          _tf_marshal_VOID__VOID,
+          0, NULL, NULL, NULL,
           G_TYPE_NONE, 0);
 }
 
@@ -255,6 +248,7 @@ _tf_content_start_sending (TfContent *self)
   GValue sending_success_val = {0,};
   gboolean sending_success;
 
+
   if (self->sending_count)
     {
       self->sending_count ++;
@@ -289,9 +283,12 @@ _tf_content_stop_sending (TfContent *self)
   self->sending_count --;
 
   if (self->sending_count == 0)
-    g_signal_emit (self, signals[SIGNAL_STOP_SENDING], 0);
+    {
+      g_signal_emit (self, signals[SIGNAL_STOP_SENDING], 0);
+    }
 }
 
+
 void
 _tf_content_emit_src_pad_added (TfContent *self, guint handle,
     FsStream *stream, GstPad *pad, FsCodec *codec)
@@ -303,24 +300,25 @@ _tf_content_emit_src_pad_added (TfContent *self, guint handle,
 /**
  * tf_content_error_literal:
  * @content: a #TfContent
- * @reason: the reason (a #TfContentRemovalReason)
- * @detailed_reason: The detailled error (as a DBus name)
  * @message: error Message
  *
- * Send an error to the Content to the CM, the effect is most likely that the
- * content will be removed.
+ * Send a fatal streaming error to the Content to the CM, the effect is most
+ * likely that the content will be removed.
+ *
+ * Rename to: tf_content_error
  */
 
 void
 tf_content_error_literal (TfContent *content,
-    guint reason, /* TfFutureContentRemovalReason */
-    const gchar *detailed_reason,
     const gchar *message)
 {
    TfContentClass *klass = TF_CONTENT_GET_CLASS (content);
 
+   g_return_if_fail (content != NULL);
+   g_return_if_fail (message != NULL);
+
    if (klass->content_error)
-     klass->content_error (content, reason, detailed_reason, message);
+     klass->content_error (content, message);
    else
      GST_WARNING ("content_error not defined in class: %s", message);
 }
@@ -328,30 +326,29 @@ tf_content_error_literal (TfContent *content,
 /**
  * tf_content_error:
  * @content: a #TfContent
- * @reason: the reason (a #TfContentRemovalReason)
- * @detailed_reason: The detailled error (as a DBus name)
  * @message_format: error Message with printf style formatting
  * @...:  Parameters to insert into the @message_format string
  *
- * Send an error to the Content to the CM, the effect is most likely that the
- * content will be removed.
+ * Send a fatal streaming error to the Content to the CM, the effect is most
+ * likely that the content will be removed.
  */
 
 void
 tf_content_error (TfContent *content,
-    guint reason, /* TfFutureContentRemovalReason */
-    const gchar *detailed_reason,
     const gchar *message_format,
     ...)
 {
   gchar *message;
   va_list valist;
 
+  g_return_if_fail (content != NULL);
+  g_return_if_fail (message_format != NULL);
+
   va_start (valist, message_format);
   message = g_strdup_vprintf (message_format, valist);
   va_end (valist);
 
-  tf_content_error_literal (content, reason, detailed_reason, message);
+  tf_content_error_literal (content, message);
   g_free (message);
 }
 
@@ -373,6 +370,8 @@ tf_content_iterate_src_pads (TfContent *content, guint *handles,
 {
   TfContentClass *klass = TF_CONTENT_GET_CLASS (content);
 
+  g_return_val_if_fail (content != NULL, NULL);
+
   if (klass->iterate_src_pads)
     return klass->iterate_src_pads (content, handles, handle_count);
   else
@@ -419,6 +418,133 @@ void
 _tf_content_stop_receiving (TfContent *self, guint *handles,
     guint handle_count)
 {
-  g_signal_emit (self, signals[SIGNAL_STOP_SENDING], 0, handles,
+  g_debug ("Requesting that the application stop receiving");
+  g_signal_emit (self, signals[SIGNAL_STOP_RECEIVING], 0, handles,
       handle_count);
 }
+
+
+/**
+ * tf_content_sending_failed_literal:
+ * @content: a #TfContent
+ * @message: The error message
+ *
+ * Informs the Connection Manager that sending has failed for this
+ * content. This is a transient error and it may or not not end the Content
+ * and the call.
+ *
+ * Rename to: tf_content_sending_failed
+ */
+
+void
+tf_content_sending_failed_literal (TfContent *content,
+    const gchar *message)
+{
+  TfContentClass *klass = TF_CONTENT_GET_CLASS (content);
+
+  g_return_if_fail (content != NULL);
+  g_return_if_fail (message != NULL);
+
+   if (klass->content_error)
+     klass->sending_failed (content, message);
+   else
+     GST_WARNING ("sending_failed not defined in class, ignoring error: %s",
+         message);
+}
+
+/**
+ * tf_content_sending_failed:
+ * @content: a #TfContent
+ * @message_format: Message with printf style formatting
+ * @...:  Parameters to insert into the @message_format string
+ *
+ * Informs the Connection Manager that sending has failed for this
+ * content. This is a transient error and it may or not not end the Content
+ * and the call.
+ */
+
+void
+tf_content_sending_failed (TfContent *content,
+    const gchar *message_format, ...)
+{
+  gchar *message;
+  va_list valist;
+
+  g_return_if_fail (content != NULL);
+  g_return_if_fail (message_format != NULL);
+
+  va_start (valist, message_format);
+  message = g_strdup_vprintf (message_format, valist);
+  va_end (valist);
+
+  tf_content_sending_failed_literal (content, message);
+  g_free (message);
+}
+
+/**
+ * tf_content_receiving_failed_literal:
+ * @content: a #TfContent
+ * @handles: an array of #guint representing Telepathy handles, may be %NULL
+ * @handle_count: the numner of handles in @handles
+ * @message: The error message
+ *
+ * Informs the Connection Manager that receiving has failed for this
+ * content. This is a transient error and it may or not not end the Content
+ * and the call.
+ *
+ * If handles are not specific, it assumes that it is valid for all handles.
+ *
+ * Rename to: tf_content_receiving_failed
+ */
+
+void
+tf_content_receiving_failed_literal (TfContent *content,
+    guint *handles, guint handle_count,
+    const gchar *message)
+{
+  TfContentClass *klass = TF_CONTENT_GET_CLASS (content);
+
+  g_return_if_fail (content != NULL);
+  g_return_if_fail (message != NULL);
+
+  if (klass->content_error)
+    klass->receiving_failed (content, handles, handle_count, message);
+  else
+    GST_WARNING ("receiving_failed not defined in class, ignoring error: %s",
+        message);
+}
+
+
+/**
+ * tf_content_receiving_failed:
+ * @content: a #TfContent
+ * @handles: an array of #guint representing Telepathy handles, may be %NULL
+ * @handle_count: the numner of handles in @handles
+ * @message_format: Message with printf style formatting
+ * @...:  Parameters to insert into the @message_format string
+ *
+ * Informs the Connection Manager that receiving has failed for this
+ * content. This is a transient error and it may or not not end the Content
+ * and the call.
+ *
+ * If handles are not specific, it assumes that it is valid for all handles.
+ */
+
+void
+tf_content_receiving_failed (TfContent *content,
+    guint *handles, guint handle_count,
+    const gchar *message_format, ...)
+{
+  gchar *message;
+  va_list valist;
+
+  g_return_if_fail (content != NULL);
+  g_return_if_fail (message_format != NULL);
+
+  va_start (valist, message_format);
+  message = g_strdup_vprintf (message_format, valist);
+  va_end (valist);
+
+  tf_content_receiving_failed_literal (content, handles, handle_count, message);
+  g_free (message);
+}
