@@ -24,9 +24,10 @@
  *
  * This class handles the
  * org.freedesktop.Telepathy.Channel.Interface.MediaSignalling on a
- * channel using Farsight2.
+ * channel using Farstream.
  */
 
+#include "config.h"
 
 #include "media-signalling-channel.h"
 
@@ -35,31 +36,8 @@
 
 #include "stream.h"
 #include "session-priv.h"
-#include "stream-priv.h"
-#include "tf-signals-marshal.h"
 
 
-struct _TfMediaSignallingChannel {
-  GObject parent;
-
-  TpChannel *channel_proxy;
-
-  TfNatProperties nat_props;
-  guint prop_id_nat_traversal;
-  guint prop_id_stun_server;
-  guint prop_id_stun_port;
-  guint prop_id_gtalk_p2p_relay_token;
-
-  /* sessions is NULL until we've had a reply from GetSessionHandlers */
-  TfSession *session;
-  gboolean got_sessions;
-  GPtrArray *streams;
-};
-
-struct _TfMediaSignallingChannelClass{
-  GObjectClass parent_class;
-};
-
 
 G_DEFINE_TYPE (TfMediaSignallingChannel, tf_media_signalling_channel,
     G_TYPE_OBJECT);
@@ -121,8 +99,7 @@ tf_media_signalling_channel_class_init (TfMediaSignallingChannelClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          g_cclosure_marshal_VOID__OBJECT,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 1, TF_TYPE_STREAM);
 
   /**
@@ -142,8 +119,7 @@ tf_media_signalling_channel_class_init (TfMediaSignallingChannelClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          _tf_marshal_VOID__OBJECT,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 1, FS_TYPE_CONFERENCE);
 
   /**
@@ -162,8 +138,7 @@ tf_media_signalling_channel_class_init (TfMediaSignallingChannelClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          _tf_marshal_VOID__OBJECT_OBJECT,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 2, FS_TYPE_CONFERENCE, FS_TYPE_PARTICIPANT);
 
 
@@ -172,8 +147,7 @@ tf_media_signalling_channel_class_init (TfMediaSignallingChannelClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          _tf_marshal_BOXED__UINT,
+          NULL, NULL, NULL,
           FS_TYPE_CODEC_LIST, 1, G_TYPE_UINT);
 }
 
@@ -475,8 +449,8 @@ new_stream_cb (TfSession *session,
       &local_codec_config);
 
   g_object_get (session,
-      "farsight-conference", &fs_conference,
-      "farsight-participant", &fs_participant,
+      "farstream-conference", &fs_conference,
+      "farstream-participant", &fs_participant,
       NULL);
 
   stream = _tf_stream_new ((gpointer) self, fs_conference,
@@ -568,7 +542,7 @@ add_session (TfMediaSignallingChannel *self,
       G_CALLBACK (session_invalidated_cb), self);
 
   g_object_get (self->session,
-      "farsight-conference", &conf,
+      "farstream-conference", &conf,
       NULL);
 
   g_signal_emit (self, signals[SESSION_CREATED], 0, conf);
