@@ -25,21 +25,20 @@
  *
  * This class handles the
  * org.freedesktop.Telepathy.Channel.Interface.MediaSignalling on a
- * channel using Farsight2.
+ * channel using Farstream.
  */
 
-
 #include "media-signalling-content.h"
 
-#include <gst/farsight/fs-conference-iface.h>
-#include <gst/farsight/fs-utils.h>
+#include <farstream/fs-conference.h>
+#include <farstream/fs-utils.h>
+
+#include <telepathy-glib/proxy-subclass.h>
+#include <telepathy-glib/util.h>
 
 #include <stdarg.h>
 #include <string.h>
 
-#include <telepathy-glib/proxy-subclass.h>
-
-#include "tf-signals-marshal.h"
 #include "utils.h"
 
 
@@ -86,8 +85,6 @@ tf_media_signalling_content_get_property (GObject    *object,
     GParamSpec *pspec);
 
 static void tf_media_signalling_content_error (TfContent *content,
-    guint reason, /* TfFutureContentRemovalReason */
-    const gchar *detailed_reason,
     const gchar *message);
 
 static GstIterator * tf_media_signalling_content_iterate_src_pads (
@@ -162,11 +159,11 @@ tf_media_signalling_content_get_property (GObject    *object,
       break;
     case PROP_FS_CONFERENCE:
       g_object_get_property (G_OBJECT (self->stream),
-          "farsight-conference", value);
+          "farstream-conference", value);
       break;
     case PROP_FS_SESSION:
       g_object_get_property (G_OBJECT (self->stream),
-          "farsight-session", value);
+          "farstream-session", value);
       break;
     case PROP_SINK_PAD:
       g_object_get_property (G_OBJECT (self->stream),
@@ -213,8 +210,8 @@ tf_media_signalling_content_new (
       G_CALLBACK (restart_source), G_OBJECT (self), 0);
 
   g_object_get (stream,
-      "farsight-conference", &conf,
-      "farsight-session", &session,
+      "farstream-conference", &conf,
+      "farstream-session", &session,
       NULL);
 
   codec_prefs = fs_utils_get_default_codec_preferences (conf);
@@ -235,7 +232,7 @@ src_pad_added (TfStream *stream, GstPad *pad, FsCodec *codec,
 {
   FsStream *fs_stream;
 
-  g_object_get (stream, "farsight-stream", &fs_stream, NULL);
+  g_object_get (stream, "farstream-stream", &fs_stream, NULL);
   _tf_content_emit_src_pad_added (TF_CONTENT (self), self->handle, fs_stream,
       pad, codec);
   g_object_unref (fs_stream);
@@ -247,18 +244,20 @@ request_resource (TfStream *stream, guint direction,
 {
   if (direction & TP_MEDIA_STREAM_DIRECTION_SEND)
     return _tf_content_start_sending (TF_CONTENT (self));
-
-  if (!self->receiving && direction & TP_MEDIA_STREAM_DIRECTION_RECEIVE)
+  else if (direction & TP_MEDIA_STREAM_DIRECTION_RECEIVE)
     {
-      guint handles[2] = { self->handle, 0};
+      if (!self->receiving)
+        {
+          guint handles[2] = { self->handle, 0};
 
-      self->receiving = _tf_content_start_receiving (TF_CONTENT (self),
-          handles, 1);
+          self->receiving = _tf_content_start_receiving (TF_CONTENT (self),
+              handles, 1);
+        }
 
       return self->receiving;
     }
-
-  return FALSE;
+  else
+    g_assert_not_reached ();
 }
 
 
@@ -286,23 +285,11 @@ restart_source (TfStream *stream, TfMediaSignallingContent *self)
 
 static void
 tf_media_signalling_content_error (TfContent *content,
-    guint reason, /* TfFutureContentRemovalReason */
-    const gchar *detailed_reason,
     const gchar *message)
 {
   TfMediaSignallingContent *self = TF_MEDIA_SIGNALLING_CONTENT (content);
-  TpMediaStreamError stream_error;
 
-  switch (reason)
-    {
-    case TF_FUTURE_CONTENT_REMOVAL_REASON_ERROR:
-      stream_error = TP_MEDIA_STREAM_ERROR_MEDIA_ERROR;
-      break;
-    default:
-      stream_error = TP_MEDIA_STREAM_ERROR_UNKNOWN;
-    }
-
-  tf_stream_error (self->stream,  stream_error, message);
+  tf_stream_error (self->stream, TP_MEDIA_STREAM_ERROR_MEDIA_ERROR, message);
 }
 
 static GstIterator *
@@ -316,8 +303,8 @@ tf_media_signalling_content_iterate_src_pads (TfContent *content,
 
   g_return_val_if_fail (handle_count <= 1, NULL);
 
-  g_object_get (self->stream, "farsight-stream", &fs_stream, NULL);
-  iter = fs_stream_get_src_pads_iterator (fs_stream);
+  g_object_get (self->stream, "farstream-stream", &fs_stream, NULL);
+  iter = fs_stream_iterate_src_pads (fs_stream);
   g_object_unref (fs_stream);
 
   return iter;
