@@ -22,7 +22,7 @@
  * SECTION:stream
  * @short_description: Handles a media Stream
  *
- * These objects handle media streams and wrap the appropriate Farsight 2
+ * These objects handle media streams and wrap the appropriate Farstream
  * objects. It is used to interact on a stream level with the other parts
  * of the media pipeline and the proper UI.
  */
@@ -41,14 +41,12 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
 
-#include <gst/farsight/fs-conference-iface.h>
-#include <gst/farsight/fs-rtp.h>
-#include <gst/farsight/fs-utils.h>
+#include <farstream/fs-conference.h>
+#include <farstream/fs-rtp.h>
+#include <farstream/fs-utils.h>
 
 #include "stream.h"
-#include "stream-priv.h"
-#include "channel.h"
-#include "tf-signals-marshal.h"
+#include "media-signalling-channel.h"
 #include "utils.h"
 
 G_DEFINE_TYPE (TfStream, tf_stream, G_TYPE_OBJECT);
@@ -83,7 +81,7 @@ struct DtmfEvent {
 
 struct _TfStreamPrivate
 {
-  TfChannel *channel;
+  TfMediaSignallingChannel *channel;
   FsConference *fs_conference;
   FsParticipant *fs_participant;
   FsSession *fs_session;
@@ -121,7 +119,7 @@ struct _TfStreamPrivate
 
   GQueue events_to_send;
 
-  gint sending_telephony_event;
+  gboolean sending_telephony_event;
 };
 
 enum
@@ -141,10 +139,10 @@ static guint signals[SIGNAL_COUNT] = {0};
 enum
 {
   PROP_CHANNEL = 1,
-  PROP_FARSIGHT_CONFERENCE,
-  PROP_FARSIGHT_SESSION,
-  PROP_FARSIGHT_STREAM,
-  PROP_FARSIGHT_PARTICIPANT,
+  PROP_FARSTREAM_CONFERENCE,
+  PROP_FARSTREAM_SESSION,
+  PROP_FARSTREAM_STREAM,
+  PROP_FARSTREAM_PARTICIPANT,
   PROP_PROXY,
   PROP_STREAM_ID,
   PROP_MEDIA_TYPE,
@@ -252,7 +250,7 @@ tf_stream_init (TfStream *self)
   g_static_mutex_init (&priv->mutex);
   priv->has_resource = TP_MEDIA_STREAM_DIRECTION_NONE;
   priv->current_state = TP_MEDIA_STREAM_STATE_DISCONNECTED;
-  priv->sending_telephony_event = -1;
+  priv->sending_telephony_event = FALSE;
 
   g_queue_init (&priv->events_to_send);
 }
@@ -270,16 +268,16 @@ tf_stream_get_property (GObject    *object,
     case PROP_CHANNEL:
       g_value_set_object (value, self->priv->channel);
       break;
-    case PROP_FARSIGHT_CONFERENCE:
+    case PROP_FARSTREAM_CONFERENCE:
       g_value_set_object (value, self->priv->fs_conference);
       break;
-    case PROP_FARSIGHT_PARTICIPANT:
+    case PROP_FARSTREAM_PARTICIPANT:
       g_value_set_object (value, self->priv->fs_participant);
       break;
-    case PROP_FARSIGHT_SESSION:
+    case PROP_FARSTREAM_SESSION:
       g_value_set_object (value, self->priv->fs_session);
       break;
-    case PROP_FARSIGHT_STREAM:
+    case PROP_FARSTREAM_STREAM:
       g_value_set_object (value, self->priv->fs_stream);
       break;
     case PROP_PROXY:
@@ -332,13 +330,13 @@ tf_stream_set_property (GObject      *object,
     {
     case PROP_CHANNEL:
       self->priv->channel =
-          TF_CHANNEL (g_value_get_object (value));
+          TF_MEDIA_SIGNALLING_CHANNEL (g_value_get_object (value));
       break;
-    case PROP_FARSIGHT_CONFERENCE:
+    case PROP_FARSTREAM_CONFERENCE:
       self->priv->fs_conference =
           FS_CONFERENCE (g_value_dup_object (value));
       break;
-    case PROP_FARSIGHT_PARTICIPANT:
+    case PROP_FARSTREAM_PARTICIPANT:
       self->priv->fs_participant =
           FS_PARTICIPANT (g_value_dup_object (value));
       break;
@@ -430,7 +428,7 @@ tf_stream_dispose (GObject *object)
       tf_stream_free_resource (stream,
           TP_MEDIA_STREAM_DIRECTION_RECEIVE);
 
-      g_object_run_dispose (G_OBJECT (priv->fs_stream));
+      fs_stream_destroy (priv->fs_stream);
       g_object_unref (priv->fs_stream);
 
       tf_stream_free_resource (stream,
@@ -441,7 +439,7 @@ tf_stream_dispose (GObject *object)
 
   if (priv->fs_session)
     {
-      g_object_run_dispose (G_OBJECT (priv->fs_session));
+      fs_session_destroy (priv->fs_session);
       g_object_unref (priv->fs_session);
       priv->fs_session = NULL;
     }
@@ -506,36 +504,36 @@ tf_stream_class_init (TfStreamClass *klass)
       g_param_spec_object ("channel",
           "Telepathy channel",
           "The TfChannel this stream is in",
-          TF_TYPE_CHANNEL,
+          TF_TYPE_MEDIA_SIGNALLING_CHANNEL,
           G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (object_class, PROP_FARSIGHT_CONFERENCE,
-      g_param_spec_object ("farsight-conference",
-          "Farsight conference",
-          "The Farsight conference this stream will "
+  g_object_class_install_property (object_class, PROP_FARSTREAM_CONFERENCE,
+      g_param_spec_object ("farstream-conference",
+          "Farstream conference",
+          "The Farstream conference this stream will "
           "create streams within.",
           FS_TYPE_CONFERENCE,
           G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (object_class, PROP_FARSIGHT_PARTICIPANT,
-      g_param_spec_object ("farsight-participant",
-          "Farsight participant",
-          "The Farsight participant this stream will "
+  g_object_class_install_property (object_class, PROP_FARSTREAM_PARTICIPANT,
+      g_param_spec_object ("farstream-participant",
+          "Farstream participant",
+          "The Farstream participant this stream will "
           "create streams for.",
           FS_TYPE_PARTICIPANT,
           G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (object_class, PROP_FARSIGHT_SESSION,
-      g_param_spec_object ("farsight-session",
-          "Farsight session",
-          "The Farsight session",
+  g_object_class_install_property (object_class, PROP_FARSTREAM_SESSION,
+      g_param_spec_object ("farstream-session",
+          "Farstream session",
+          "The Farstream session",
           FS_TYPE_SESSION,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (object_class, PROP_FARSIGHT_STREAM,
-      g_param_spec_object ("farsight-stream",
-          "Farsight stream",
-          "The Farsight stream",
+  g_object_class_install_property (object_class, PROP_FARSTREAM_STREAM,
+      g_param_spec_object ("farstream-stream",
+          "Farstream stream",
+          "The Farstream stream",
           FS_TYPE_STREAM,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
@@ -640,8 +638,7 @@ tf_stream_class_init (TfStreamClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          g_cclosure_marshal_VOID__VOID,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 0);
 
   /**
@@ -656,8 +653,7 @@ tf_stream_class_init (TfStreamClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          g_cclosure_marshal_VOID__VOID,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 0);
 
   /**
@@ -679,8 +675,7 @@ tf_stream_class_init (TfStreamClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          g_signal_accumulator_true_handled, NULL,
-          _tf_marshal_BOOLEAN__UINT,
+          g_signal_accumulator_true_handled, NULL, NULL,
           G_TYPE_BOOLEAN, 1, G_TYPE_UINT);
 
   /**
@@ -698,8 +693,7 @@ tf_stream_class_init (TfStreamClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          g_cclosure_marshal_VOID__UINT,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 1, G_TYPE_UINT);
 
   /**
@@ -717,8 +711,7 @@ tf_stream_class_init (TfStreamClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          _tf_marshal_VOID__OBJECT_BOXED,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 2, GST_TYPE_PAD, FS_TYPE_CODEC);
 
 
@@ -735,8 +728,7 @@ tf_stream_class_init (TfStreamClass *klass)
           G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST,
           0,
-          NULL, NULL,
-          _tf_marshal_VOID__VOID,
+          NULL, NULL, NULL,
           G_TYPE_NONE, 0);
 }
 
@@ -762,6 +754,7 @@ get_all_properties_cb (TpProxy *proxy,
   guint i;
   gboolean do_controlling = FALSE;
   GList *rtp_header_extensions;
+  gboolean res = FALSE;
 
   if (dbus_error &&
       !(dbus_error->domain == DBUS_GERROR &&
@@ -1056,11 +1049,12 @@ get_all_properties_cb (TpProxy *proxy,
   stream->priv->fs_stream = fs_session_new_stream (stream->priv->fs_session,
       stream->priv->fs_participant,
       FS_DIRECTION_NONE,
-      transmitter,
-      n_args,
-      params,
       &myerror);
 
+  if (stream->priv->fs_stream)
+    res = fs_stream_set_transmitter (stream->priv->fs_stream,
+        transmitter, params, n_args, &myerror);
+
   for (i = 0; i < n_args; i++)
     g_value_unset (&params[i].value);
 
@@ -1072,6 +1066,15 @@ get_all_properties_cb (TpProxy *proxy,
       return;
     }
 
+  if (!res)
+    {
+      tf_stream_error (stream, fserror_to_tperror (myerror), myerror->message);
+      WARNING (stream, "Could not set transmitter for stream: %s",
+          myerror->message);
+      g_clear_error (&myerror);
+      return;
+    }
+
   rtp_header_extensions =
       fs_utils_get_default_rtp_header_extension_preferences (
           GST_ELEMENT (stream->priv->fs_conference),
@@ -1295,7 +1298,7 @@ tp_transports_to_fs (const gchar* foundation, const GPtrArray *transports)
           type = FS_CANDIDATE_TYPE_RELAY;
           break;
         default:
-          g_critical ("%s: FarsightTransportInfo.proto has an invalid value",
+          g_critical ("%s: FarstreamTransportInfo.proto has an invalid value",
               G_STRFUNC);
           type = FS_CANDIDATE_TYPE_HOST;
         }
@@ -1309,7 +1312,7 @@ tp_transports_to_fs (const gchar* foundation, const GPtrArray *transports)
           proto = FS_NETWORK_PROTOCOL_TCP;
           break;
         default:
-          g_critical ("%s: FarsightTransportInfo.proto has an invalid value",
+          g_critical ("%s: FarstreamTransportInfo.proto has an invalid value",
               G_STRFUNC);
           proto = FS_NETWORK_PROTOCOL_UDP;
         }
@@ -1347,7 +1350,7 @@ fs_network_proto_to_tp (FsNetworkProtocol proto, gboolean *valid)
   case FS_NETWORK_PROTOCOL_TCP:
     return TP_MEDIA_STREAM_BASE_PROTO_TCP;
   default:
-    g_critical ("%s: FarsightTransportInfo.proto has an invalid value",
+    g_critical ("%s: FarstreamTransportInfo.proto has an invalid value",
         G_STRFUNC);
     if (valid != NULL)
       *valid = FALSE;
@@ -1370,7 +1373,7 @@ fs_candidate_type_to_tp (FsCandidateType type, gboolean *valid)
   case FS_CANDIDATE_TYPE_RELAY:
     return TP_MEDIA_STREAM_TRANSPORT_TYPE_RELAY;
   default:
-    g_critical ("%s: FarsightTransportInfo.proto has an invalid value",
+    g_critical ("%s: FarstreamTransportInfo.proto has an invalid value",
         G_STRFUNC);
     if (valid != NULL)
       *valid = FALSE;
@@ -1410,7 +1413,7 @@ fs_candidate_to_tp_array (const FsCandidate *candidate)
 }
 
 /*
- * Small helper function to help converting a list of FarsightCodecs
+ * Small helper function to help converting a list of FarstreamCodecs
  * to a Telepathy codec list.
  */
 static GPtrArray *
@@ -1438,7 +1441,7 @@ fs_codecs_to_tp (TfStream *stream,
           type = TP_MEDIA_STREAM_TYPE_VIDEO;
           break;
         default:
-          g_critical ("%s: FarsightCodec [%d, %s]'s media_type has an invalid value",
+          g_critical ("%s: FarstreamCodec [%d, %s]'s media_type has an invalid value",
               G_STRFUNC, fsc->id, fsc->encoding_name);
           return NULL;
       }
@@ -1454,13 +1457,6 @@ fs_codecs_to_tp (TfStream *stream,
                                g_strdup (param->value));
         }
 
-      if (fsc->ABI.ABI.ptime)
-        g_hash_table_insert (params, g_strdup ("ptime"),
-            g_strdup_printf ("%u", fsc->ABI.ABI.ptime));
-      if (fsc->ABI.ABI.maxptime)
-        g_hash_table_insert (params, g_strdup ("maxptime"),
-            g_strdup_printf ("%u", fsc->ABI.ABI.maxptime));
-
       g_value_init (&codec, TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CODEC);
       g_value_take_boxed (&codec,
           dbus_g_type_specialized_construct (TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CODEC));
@@ -1499,7 +1495,7 @@ add_remote_candidate (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
 
   fscandidates = tp_transports_to_fs (candidate, transports);
 
-  if (!fs_stream_set_remote_candidates (self->priv->fs_stream,
+  if (!fs_stream_add_remote_candidates (self->priv->fs_stream,
           fscandidates, &error))
     tf_stream_error (self, fserror_to_tperror (error), error->message);
 
@@ -1555,6 +1551,7 @@ set_remote_candidate_list (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
   guint i;
   GList *fs_candidates = NULL;
   GError *error = NULL;
+  gboolean ret;
 
   for (i = 0; i < candidates->len; i++)
     {
@@ -1577,9 +1574,17 @@ set_remote_candidate_list (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
           tp_transports_to_fs (foundation, transports));
     }
 
-  if (!fs_stream_set_remote_candidates (self->priv->fs_stream,
-                  fs_candidates, &error))
-    tf_stream_error (self, fserror_to_tperror (error), error->message);
+  ret = fs_stream_add_remote_candidates (self->priv->fs_stream,
+      fs_candidates, &error);
+  if (!ret && error &&
+      error->domain == FS_ERROR && error->code == FS_ERROR_NOT_IMPLEMENTED)
+  {
+    g_clear_error (&error);
+    ret = fs_stream_force_remote_candidates (self->priv->fs_stream,
+        fs_candidates, &error);
+  }
+  if (!ret)
+    tf_stream_error (self, fserror_to_tperror (error),  error->message);
 
   g_clear_error (&error);
   fs_candidate_list_destroy (fs_candidates);
@@ -1590,29 +1595,7 @@ fill_fs_params (gpointer key, gpointer value, gpointer user_data)
 {
   FsCodec *codec = user_data;
 
-  if (!strcmp (key, "ptime") && atoi (value) > 0)
-    codec->ABI.ABI.ptime = atoi (value);
-  else if  (!strcmp (key, "maxptime") && atoi (value) > 0)
-    codec->ABI.ABI.maxptime = atoi (value);
-  else
-    fs_codec_add_optional_parameter (codec, key, value);
-}
-
-static FsStreamDirection
-tpdirection_to_fsdirection (TpMediaStreamDirection dir)
-{
-  switch (dir) {
-  case TP_MEDIA_STREAM_DIRECTION_NONE:
-    return FS_DIRECTION_NONE;
-  case TP_MEDIA_STREAM_DIRECTION_SEND:
-    return FS_DIRECTION_SEND;
-  case TP_MEDIA_STREAM_DIRECTION_RECEIVE:
-    return FS_DIRECTION_RECV;
-  case TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL:
-    return FS_DIRECTION_BOTH;
-  default:
-    g_assert_not_reached ();
-  }
+  fs_codec_add_optional_parameter (codec, key, value);
 }
 
 static void
@@ -1678,7 +1661,7 @@ set_remote_codecs (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
                   TP_ARRAY_TYPE_RTCP_FEEDBACK_MESSAGE_LIST);
 
               val = g_value_array_get_nth (message_props, 0);
-              fs_codec->ABI.ABI.minimum_reporting_interval =
+              fs_codec->minimum_reporting_interval =
                   g_value_get_uint (val);
 
               val = g_value_array_get_nth (message_props, 1);
@@ -2010,32 +1993,28 @@ start_telephony_event (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
 
   DEBUG (self, "called with event %u", event);
 
-  if (self->priv->sending_telephony_event != -1)
+  if (self->priv->sending_telephony_event)
     {
       WARNING (self, "start new telephony event without stopping the"
           " previous one first");
-      if (!fs_session_stop_telephony_event (self->priv->fs_session,
-              self->priv->sending_telephony_event))
+      if (!fs_session_stop_telephony_event (self->priv->fs_session))
         WARNING (self, "stopping event failed");
     }
 
   /* this week, volume is 8, for the sake of argument... */
 
-  if (!fs_session_start_telephony_event (self->priv->fs_session, event, 8,
-          FS_DTMF_METHOD_AUTO))
+  if (!fs_session_start_telephony_event (self->priv->fs_session, event, 8))
     WARNING (self, "sending event %u failed", event);
-  self->priv->sending_telephony_event = FS_DTMF_METHOD_AUTO;
+  self->priv->sending_telephony_event = TRUE;
 }
 
 static gboolean
 check_codecs_for_telephone_event (TfStream *self, GList **codecs,
-    FsCodec *send_codec, guint codecid)
+    FsCodec *send_codec, gint codecid)
 {
   GList *item = NULL;
-  gboolean found = FALSE;
   GError *error = NULL;
-
- again:
+  gboolean changed = FALSE;
 
   for (item = *codecs; item; item = item->next)
     {
@@ -2044,29 +2023,25 @@ check_codecs_for_telephone_event (TfStream *self, GList **codecs,
       if (!g_ascii_strcasecmp (codec->encoding_name, "telephone-event") &&
           send_codec->clock_rate == codec->clock_rate)
         {
-          if (found)
-            {
-              *codecs = g_list_delete_link (*codecs, item);
-              goto again;
-            }
-          else if (codecid == (guint) codec->id)
-            {
-              return TRUE;            }
+          if (codecid < 0 || codecid == codec->id)
+            return TRUE;
           else
-            {
-              codec->id = codecid;
-            }
+            codec->id = codecid;
+          changed = TRUE;
+          break;
         }
     }
 
-  if (!found)
+  if (codecid < 0)
+    return FALSE;
+
+  if (!changed)
     {
       FsCodec *codec = fs_codec_new (codecid, "telephone-event",
           FS_MEDIA_TYPE_AUDIO, send_codec->clock_rate);
 
       *codecs = g_list_append (*codecs, codec);
     }
-
   if (!fs_stream_set_remote_codecs (self->priv->fs_stream, *codecs, &error))
     {
       /*
@@ -2102,12 +2077,11 @@ start_named_telephony_event (TpMediaStreamHandler *proxy,
 
   if (check_codecs_for_telephone_event (self, &codecs, send_codec, codecid))
     {
-      if (self->priv->sending_telephony_event != -1)
+      if (self->priv->sending_telephony_event)
         {
           WARNING (self, "start new telephony event without stopping the"
               " previous one first");
-          if (!fs_session_stop_telephony_event (self->priv->fs_session,
-                  self->priv->sending_telephony_event))
+          if (!fs_session_stop_telephony_event (self->priv->fs_session))
             WARNING (self, "stopping event failed");
         }
 
@@ -2115,9 +2089,9 @@ start_named_telephony_event (TpMediaStreamHandler *proxy,
       DEBUG (self, "Sending named telephony event %d with pt %d",
           event, codecid);
       if (!fs_session_start_telephony_event (self->priv->fs_session,
-              event, 8, FS_DTMF_METHOD_RTP_RFC4733))
+              event, 8))
         WARNING (self, "sending event %u failed", event);
-      self->priv->sending_telephony_event = FS_DTMF_METHOD_RTP_RFC4733;
+      self->priv->sending_telephony_event = TRUE;
     }
   else
     {
@@ -2140,26 +2114,45 @@ start_sound_telephony_event (TpMediaStreamHandler *proxy, guchar event,
     gpointer user_data, GObject *object)
 {
   TfStream *self = TF_STREAM (object);
+  FsCodec *send_codec = NULL;
+  GList *codecs = NULL;
 
   g_assert (self->priv->fs_session != NULL);
 
   DEBUG (self, "called with event %u", event);
 
-  if (self->priv->sending_telephony_event != -1)
+  g_object_get (self->priv->fs_session,
+      "current-send-codec", &send_codec,
+      "codecs", &codecs,
+      NULL);
+
+  if (send_codec == NULL)
+    goto out;
+
+  if (check_codecs_for_telephone_event (self, &codecs, send_codec, -1))
+    {
+      WARNING (self, "Tried to do sound event while telephone-event is set,"
+          " ignoring");
+      goto out;
+    }
+
+  if (self->priv->sending_telephony_event)
     {
       WARNING (self, "start new telephony event without stopping the"
           " previous one first");
-      if (!fs_session_stop_telephony_event (self->priv->fs_session,
-              self->priv->sending_telephony_event))
+      if (!fs_session_stop_telephony_event (self->priv->fs_session))
         WARNING (self, "stopping event failed");
     }
 
   /* this week, volume is 8, for the sake of argument... */
 
-  if (!fs_session_start_telephony_event (self->priv->fs_session, event, 8,
-          FS_DTMF_METHOD_IN_BAND))
+  if (!fs_session_start_telephony_event (self->priv->fs_session, event, 8))
     WARNING (self, "sending sound event %u failed", event);
-  self->priv->sending_telephony_event = FS_DTMF_METHOD_IN_BAND;
+  self->priv->sending_telephony_event = TRUE;
+
+ out:
+  fs_codec_destroy (send_codec);
+  fs_codec_list_destroy (codecs);
 }
 
 
@@ -2174,13 +2167,11 @@ stop_telephony_event (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
 
   DEBUG (self, "called");
 
-  if (self->priv->sending_telephony_event == -1)
-      WARNING (self, "Trying to stop telephony event without having started"
-          " one");
-  self->priv->sending_telephony_event = -1;
+  if (!self->priv->sending_telephony_event)
+    WARNING (self, "Trying to stop telephony event without having started one");
+  self->priv->sending_telephony_event = FALSE;
 
-  if (!fs_session_stop_telephony_event (self->priv->fs_session,
-          FS_DTMF_METHOD_AUTO))
+  if (!fs_session_stop_telephony_event (self->priv->fs_session))
     WARNING (self, "stopping event failed");
 }
 
@@ -2388,21 +2379,19 @@ cb_fs_send_codec_changed (TfStream *self,
       DEBUG (self, "Sending queued event %d with pt %d", dtmfevent->event_id,
           dtmfevent->codec_id);
       dtmfevent = g_queue_pop_head (&self->priv->events_to_send);
-      if (self->priv->sending_telephony_event != -1)
+      if (self->priv->sending_telephony_event)
         {
           WARNING (self, "start new telephony event without stopping the"
               " previous one first");
-          if (!fs_session_stop_telephony_event (self->priv->fs_session,
-                  self->priv->sending_telephony_event))
+          if (!fs_session_stop_telephony_event (self->priv->fs_session))
             WARNING (self, "stopping event failed");
         }
-      self->priv->sending_telephony_event = -1;
+      self->priv->sending_telephony_event = FALSE;
 
       if (!fs_session_start_telephony_event (self->priv->fs_session,
-              dtmfevent->event_id, 8, FS_DTMF_METHOD_RTP_RFC4733))
+              dtmfevent->event_id, 8))
         WARNING (self, "sending event %u failed", dtmfevent->event_id);
-      fs_session_stop_telephony_event (self->priv->fs_session,
-          FS_DTMF_METHOD_RTP_RFC4733);
+      fs_session_stop_telephony_event (self->priv->fs_session);
 
       g_slice_free (struct DtmfEvent, dtmfevent);
     }
@@ -2447,7 +2436,6 @@ gboolean
 _tf_stream_bus_message (TfStream *stream,
     GstMessage *message)
 {
-  const gchar *debug = NULL;
   const GstStructure *s = gst_message_get_structure (message);
 
   if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
@@ -2456,7 +2444,7 @@ _tf_stream_bus_message (TfStream *stream,
   if (!stream->priv->fs_stream || !stream->priv->fs_session)
     return FALSE;
 
-  if (gst_structure_has_name (s, "farsight-error"))
+  if (gst_structure_has_name (s, "farstream-error"))
     {
       GObject *object;
       const GValue *value = NULL;
@@ -2475,27 +2463,18 @@ _tf_stream_bus_message (TfStream *stream,
           value = gst_structure_get_value (s, "error-no");
           errorno = g_value_get_enum (value);
           msg = gst_structure_get_string (s, "error-msg");
-          debug = gst_structure_get_string (s, "debug-msg");
-
-          /*
-           * We ignore the Unknown Cname error because current signalling
-           * does no provide us with a cname
-           */
-          if (errorno == FS_ERROR_UNKNOWN_CNAME)
-            return TRUE;
-
 
           enumclass = g_type_class_ref (FS_TYPE_ERROR);
           enumvalue = g_enum_get_value (enumclass, errorno);
-          WARNING (stream, "error (%s (%d)): %s : %s",
-              enumvalue->value_nick, errorno, msg, debug);
+          WARNING (stream, "error (%s (%d)): %s",
+              enumvalue->value_nick, errorno, msg);
           g_type_class_unref (enumclass);
 
           tf_stream_error (stream, fserrorno_to_tperrorno (errorno), msg);
           return TRUE;
         }
     }
-  else if (gst_structure_has_name (s, "farsight-new-local-candidate"))
+  else if (gst_structure_has_name (s, "farstream-new-local-candidate"))
     {
       FsStream *fsstream;
       FsCandidate *candidate;
@@ -2504,16 +2483,20 @@ _tf_stream_bus_message (TfStream *stream,
       value = gst_structure_get_value (s, "stream");
       fsstream = g_value_get_object (value);
 
+      g_debug ("new local fs: %p s:%p", stream->priv->fs_stream, stream);
+
       if (fsstream != stream->priv->fs_stream)
         return FALSE;
 
       value = gst_structure_get_value (s, "candidate");
       candidate = g_value_get_boxed (value);
 
+      g_debug ("NEW LOCAL CAND");
+
       cb_fs_new_local_candidate (stream, candidate);
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-local-candidates-prepared"))
+  else if (gst_structure_has_name (s, "farstream-local-candidates-prepared"))
     {
       FsStream *fsstream;
       const GValue *value;
@@ -2521,14 +2504,18 @@ _tf_stream_bus_message (TfStream *stream,
       value = gst_structure_get_value (s, "stream");
       fsstream = g_value_get_object (value);
 
+      g_debug ("local cand prep fs: %p s:%p", stream->priv->fs_stream, stream);
+
       if (fsstream != stream->priv->fs_stream)
         return FALSE;
 
+      g_debug ("LOCAL CAND PREP");
+
       cb_fs_local_candidates_prepared (stream);
 
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-new-active-candidate-pair"))
+  else if (gst_structure_has_name (s, "farstream-new-active-candidate-pair"))
     {
       FsStream *fsstream;
       FsCandidate *local_candidate;
@@ -2550,7 +2537,7 @@ _tf_stream_bus_message (TfStream *stream,
       cb_fs_new_active_candidate_pair (stream, local_candidate, remote_candidate);
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-current-recv-codecs-changed"))
+  else if (gst_structure_has_name (s, "farstream-current-recv-codecs-changed"))
     {
       FsStream *fsstream;
       GList *codecs;
@@ -2568,7 +2555,7 @@ _tf_stream_bus_message (TfStream *stream,
       cb_fs_recv_codecs_changed (stream, codecs);
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-codecs-changed"))
+  else if (gst_structure_has_name (s, "farstream-codecs-changed"))
     {
       FsSession *fssession;
       const GValue *value;
@@ -2585,7 +2572,7 @@ _tf_stream_bus_message (TfStream *stream,
 
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-send-codec-changed"))
+  else if (gst_structure_has_name (s, "farstream-send-codec-changed"))
     {
       FsSession *fssession;
       const GValue *value;
@@ -2618,13 +2605,9 @@ _tf_stream_bus_message (TfStream *stream,
             FS_CODEC_ARGS (codec));
 
       cb_fs_send_codec_changed (stream, codec, secondary_codecs);
-
-      if (codec)
-        fs_codec_destroy (codec);
-      fs_codec_list_destroy (secondary_codecs);
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-component-state-changed"))
+  else if (gst_structure_has_name (s, "farstream-component-state-changed"))
     {
       FsStream *fsstream;
       const GValue *value;
@@ -2645,7 +2628,7 @@ _tf_stream_bus_message (TfStream *stream,
       cb_fs_component_state_changed (stream, component, fsstate);
       return TRUE;
     }
-  else if (gst_structure_has_name (s, "farsight-renegotiate"))
+  else if (gst_structure_has_name (s, "farstream-renegotiate"))
     {
       FsSession *fssession;
       const GValue *value;
@@ -2728,8 +2711,8 @@ _tf_stream_new (gpointer channel,
 
   self = g_object_new (TF_TYPE_STREAM,
       "channel", channel,
-      "farsight-conference", conference,
-      "farsight-participant", participant,
+      "farstream-conference", conference,
+      "farstream-participant", participant,
       "proxy", proxy,
       "stream-id", stream_id,
       "media-type", media_type,
@@ -2754,15 +2737,15 @@ fs_codecs_to_feedback_messages (GList *fscodecs)
     {
       FsCodec *fs_codec = item->data;
 
-      if (fs_codec->ABI.ABI.minimum_reporting_interval != G_MAXUINT ||
-          fs_codec->ABI.ABI.feedback_params)
+      if (fs_codec->minimum_reporting_interval != G_MAXUINT ||
+          fs_codec->feedback_params)
         {
           GValueArray *codec = g_value_array_new (2);
           GPtrArray *messages = g_ptr_array_new ();
           GValue *val;
           GList *item2;
 
-          for (item2 = fs_codec->ABI.ABI.feedback_params;
+          for (item2 = fs_codec->feedback_params;
                item2;
                item2 = item2->next)
             {
@@ -2791,7 +2774,7 @@ fs_codecs_to_feedback_messages (GList *fscodecs)
           g_value_array_insert (codec, 0, NULL);
           val = g_value_array_get_nth (codec, 0);
           g_value_init (val, G_TYPE_UINT);
-          g_value_set_uint (val, fs_codec->ABI.ABI.minimum_reporting_interval);
+          g_value_set_uint (val, fs_codec->minimum_reporting_interval);
 
           g_value_array_insert (codec, 1, NULL);
           val = g_value_array_get_nth (codec, 1);
@@ -2807,23 +2790,6 @@ fs_codecs_to_feedback_messages (GList *fscodecs)
 }
 
 
-static TpMediaStreamDirection
-fsdirection_to_tpdirection (FsStreamDirection dir)
-{
-  switch (dir) {
-  case FS_DIRECTION_NONE:
-    return TP_MEDIA_STREAM_DIRECTION_NONE;
-  case FS_DIRECTION_SEND:
-    return TP_MEDIA_STREAM_DIRECTION_SEND;
-  case FS_DIRECTION_RECV:
-    return TP_MEDIA_STREAM_DIRECTION_RECEIVE;
-  case FS_DIRECTION_BOTH:
-    return TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL;
-  default:
-    g_assert_not_reached ();
-  }
-}
-
 
 static GPtrArray *
 _tf_stream_get_header_extensions (TfStream *stream)
@@ -2859,7 +2825,6 @@ _tf_stream_get_header_extensions (TfStream *stream)
 void
 _tf_stream_try_sending_codecs (TfStream *stream)
 {
-  gboolean ready = FALSE;
   GList *fscodecs = NULL;
   GList *item = NULL;
   GPtrArray *tpcodecs = NULL;
@@ -2867,21 +2832,26 @@ _tf_stream_try_sending_codecs (TfStream *stream)
   GPtrArray *header_extensions = NULL;
   gboolean sent = FALSE;
   GList *resend_codecs = NULL;
+  const gchar *codecs_prop = NULL;
 
   DEBUG (stream, "called (send_local:%d send_supported:%d)",
       stream->priv->send_local_codecs, stream->priv->send_supported_codecs);
 
-  g_object_get (stream->priv->fs_session, "codecs-ready", &ready, NULL);
 
-  if (!ready && stream->priv->has_resource & TP_MEDIA_STREAM_DIRECTION_SEND)
+  if (stream->priv->has_resource & TP_MEDIA_STREAM_DIRECTION_SEND)
+    codecs_prop = "codecs";
+  else
+    codecs_prop = "codecs-without-config";
+
+  g_object_get (stream->priv->fs_session, codecs_prop, &fscodecs, NULL);
+
+  if (!fscodecs)
     {
       DEBUG (stream, "Ignoring new codecs because we're sending,"
           " but we're not ready");
       return;
     }
 
-  g_object_get (stream->priv->fs_session, "codecs", &fscodecs, NULL);
-
   for(item = fscodecs; item; item = g_list_next (item))
     {
       gchar *tmp = fs_codec_to_string (item->data);
@@ -3060,7 +3030,7 @@ cb_fs_component_state_changed (TfStream *self,
     FsStreamState fsstate)
 {
   TpMediaStreamState state;
-  const gchar *state_str;
+  const gchar *state_str = "";
 
   if (component != 1)
     return;
@@ -3074,11 +3044,10 @@ cb_fs_component_state_changed (TfStream *self,
       break;
     case FS_STREAM_STATE_GATHERING:
     case FS_STREAM_STATE_CONNECTING:
-    case FS_STREAM_STATE_CONNECTED:
-      state_str = "connecting";
       state = TP_MEDIA_STREAM_STATE_CONNECTING;
+      state_str = "connecting";
       break;
-    case FS_STREAM_STATE_READY:
+    case FS_STREAM_STATE_CONNECTED:
     default:
       state_str = "connected";
       state = TP_MEDIA_STREAM_STATE_CONNECTED;
